MAPBOX_ACCESS_TOKEN = 'pk.eyJ1IjoibWFwYm94IiwiYSI6ImNpejY4NXVycTA2emYycXBndHRqcmZ3N3gifQ.rJcFIG214AriISLbB6B5aw';
MAX_OPACITY = 1;
FIRST_IMAGE_LAYER_INDEX = 2;


// create wkt polygon
const wktFormat = new ol.format.WKT();
const polygonFeature = wktFormat.readFeature(polygon, {
    dataProjection: 'EPSG:4326',
    featureProjection: 'EPSG:3857'
});

// Create layers
const layers2 = {
    light: new ol.layer.Tile({
        name: 'mapbox',
        source: new ol.source.XYZ({
            url: 'https://api.tiles.mapbox.com/v4/mapbox.light/{z}/{x}/{y}.png?access_token=' + MAPBOX_ACCESS_TOKEN
        })
    }),

    streets: new ol.layer.Tile({
        name: 'mapbox',
        source: new ol.source.XYZ({
            url: 'http://tile.openstreetmap.org/{z}/{x}/{y}.png'
        })
    }),

    night: new ol.layer.Tile({
        name: 'mapbox',
        source: new ol.source.XYZ({
            url: 'https://api.mapbox.com/styles/v1/mapbox/navigation-guidance-night-v2/tiles/256/{z}/{x}/{y}?access_token=' + MAPBOX_ACCESS_TOKEN
        })
    }),

    cycle: new ol.layer.Tile({
        name: 'mapbox',
        source: new ol.source.XYZ({
            url: 'https://api.tiles.mapbox.com/v4/mapbox.outdoors/{z}/{x}/{y}.png?access_token=' + MAPBOX_ACCESS_TOKEN
        }),
    })
}

const light = new ol.layer.Tile({
    name: 'mapbox',
    source: new ol.source.XYZ({
        url: 'https://api.tiles.mapbox.com/v4/mapbox.light/{z}/{x}/{y}.png?access_token=' + MAPBOX_ACCESS_TOKEN
    })
});

const streets = new ol.layer.Tile({
    name: 'mapbox',
    source: new ol.source.XYZ({
        url: 'http://tile.openstreetmap.org/{z}/{x}/{y}.png'
    })
});

const night = new ol.layer.Tile({
    name: 'mapbox',
    source: new ol.source.XYZ({
        url: 'https://api.mapbox.com/styles/v1/mapbox/navigation-guidance-night-v2/tiles/256/{z}/{x}/{y}?access_token=' + MAPBOX_ACCESS_TOKEN
    })
});

const cycle = new ol.layer.Tile({
    name: 'mapbox',
    source: new ol.source.XYZ({
        url: 'https://api.tiles.mapbox.com/v4/mapbox.outdoors/{z}/{x}/{y}.png?access_token=' + MAPBOX_ACCESS_TOKEN
    }),
});

let x = '';

if (sessionStorage.selectedMap) {
    x = sessionStorage.selectedMap;
    //console.log({x})
}
else {
    x = 'light';
    //console.log({x})
}

document.getElementById(x).checked = true;
const polygonLayer = new ol.layer.Vector({
    name: 'polygon',
    source: new ol.source.Vector({
        features: [polygonFeature]
    })
});

const layers = new Map();

// create map
const map = new ol.Map({
    target: 'minimap',
    layers: [layers2[x], polygonLayer],
    view: new ol.View({
        center: ol.proj.fromLonLat([-99.19, 19.61]),
        zoom: 4,
        minZoom: 4
    })
});

map.once('postrender', function (event) {
    if (sessionStorage.showChart === "true") {
        showGraph(true)
    }
    if (sessionStorage.selectedMap) {
        currentMap = sessionStorage.selectedMap;
    }
    if (sessionStorage.TCI) {
        $('#tcicheck').prop('checked', true);
    }
    if (sessionStorage.SCL_0) {
        $('#check-no-data').prop('checked', true);
    }
    if (sessionStorage.SCL_1) {
        $('#check-saturated').prop('checked', true);
    }
    if (sessionStorage.SCL_2) {
        $('#check-shadows').prop('checked', true);
    }
    if (sessionStorage.SCL_3) {
        $('#check-cloud-shadows').prop('checked', true);
    }
    if (sessionStorage.SCL_4) {
        $('#check-vegetation').prop('checked', true);
    }
    if (sessionStorage.SCL_5) {
        $('#check-not-vegetated').prop('checked', true);
    }
    if (sessionStorage.SCL_6) {
        $('#check-water').prop('checked', true);
    }
    if (sessionStorage.SCL_7) {
        $('#check-unclassified').prop('checked', true);
    }
    if (sessionStorage.SCL_8) {
        $('#check-clouds').prop('checked', true);
    }
    if (sessionStorage.SCL_9) {
        $('#check-snow').prop('checked', true);
    }
    if (sessionStorage.layerOpacity){
        $('#opacitySlide').val(sessionStorage.layerOpacity)
        setOpacity(sessionStorage.layerOpacity);
    }

     //console.log(sessionStorage)

    if (sessionStorage.charttype) {
        change(sessionStorage.charttype);
    }
});

// fit (zoom) to polygon
map.getView().fit(polygonFeature.getGeometry());

// show first layer date
let currentDate = labels[0];
addImageLayer({
    layerDate: currentDate
});

// main function to change layers
// data = { layerDate: to change all layers by date }
// data = { layerName: to add single layer }
function addImageLayer(data) {
    if (data.layerDate) {
        currentDate = data.layerDate;
    }

    $.ajax({
        type: 'POST',
        dataType: "json",
        data: {'purchase_id': purchase_id, 'report': zone, 'date': currentDate},
        url: '/reports/tiles/tileinfo',
        success: response => {
            changeLayers(data.layerName);
            setOpacity($('#opacitySlide').val());
            map.getView().setMinZoom(response.zoom.min);
            map.getView().setMaxZoom(response.zoom.max);
        }
    });
}

// remove old layers and add layers with new date
let current = []

function changeLayers(especific_layerName) {

    const layersToChance = especific_layerName ? especific_layerName.split() : getSelectedLayersName();
    removeLayer(especific_layerName);

    layersToChance.forEach(layerName => {
        const xyz = new ol.layer.Tile({
            name: layerName,
            source: new ol.source.XYZ({
                projection: 'EPSG:3857',
                url: '/reports/tiles/' + purchase_id + '/' + zone + '/' + layerName + '/' + currentDate + '/{z}/{x}/{-y}.png',
                tilePixelRatio: 2
            })
        });

        if (layerName === 'TCI') {
            map.getLayers().insertAt(FIRST_IMAGE_LAYER_INDEX, xyz);
        } else {
            map.addLayer(xyz);
        }

        layers.set(layerName, xyz);
        sessionStorage.setItem(layerName, layerName);
    });
}

// remove all if there isn't layername
function removeLayer(layerName) {
    if (!layerName) {
        layers.forEach(layer => {
            map.removeLayer(layer);
        });
        layers.clear();
        return;
    }

    map.removeLayer(layers.get(layerName));
    layers.delete(layerName);
    sessionStorage.removeItem(layerName)
}

// set opacity to layers
function setOpacity(value) {
    layers.forEach(layer => {
        if (layer.get('name') != 'TCI') {
            layer.setOpacity(value)
        }
    })

    if(sessionStorage.layerOpacity != value){
        sessionStorage.layerOpacity = value;
    }

}


// *********** Irving ******************
//  Funciones de comparacion

//vigila cuando se cierra el menu de comparar y cuando se cierra desactiva la comparacion (si la clase de compare incluye menu-open está abierto)
var compareMenu = document.getElementById('compare');
var observer = new MutationObserver(function (mutations) {
    mutations.forEach(function (mutation) {
        if (mutation.attributeName == 'class') {
            if (compareMenu.className.indexOf("open") === -1) {  //se cerró el menu de comparacion
                exitCompare();
            }
        }
    });
});
observer.observe(compareMenu, {attributes: true, attributeOldValue: true, attributeFilter: ['class'], subtree: true});

//crea el control swipe, que sirve para comparar
var ctrl = new ol.control.Swipe();

//funcion que inicia la comparacion
//recibe la lista de fechas y por default compara la primera y la ultima
function compare(labels) {
    var swipeControl = false;

    //oculta la grafica si esta abierta
    $("#draggableChart").css('z-index', -1);
    $(".showChart").show();
    $(".hideChart").hide();
    //desactiva el botón de mostrar la gráfica
    $('#showGraph').attr('disabled', true);
    $('#chartType').attr('disabled', true);

    //quita todas las capas SCL y deja sólo TCI
    removeLayer();
    changeLayers('TCI');
    $("#tcicheck").prop("checked", true);
    $("#check-no-data").prop("checked", false);
    $("#check-saturated").prop("checked", false);
    $("#check-shadows").prop("checked", false);
    $("#check-cloud-shadows").prop("checked", false);
    $("#check-vegetation").prop("checked", false);
    $("#check-not-vegetated").prop("checked", false);
    $("#check-water").prop("checked", false);
    $("#check-unclassified").prop("checked", false);
    $("#check-clouds").prop("checked", false);
    $("#check-snow").prop("checked", false);


    map.getControls().forEach(function (control) {  //checa si ya se añadio el control de comparacion (swipe), si no lo tiene, lo agrega
        if (control instanceof ol.control.Swipe) { //si ya tiene el control agregado
            swipeControl = true;
        }
    });

    if (swipeControl == true) {  //si ya tiene el comparador no hace nada, ya esta en modo comparacion
        //map.removeControl(swipeControlObj);
    } else {  //si no tiene el comparador lo agrega
        //removeImageLayers();
        map.addControl(ctrl);
        ctrl.set('position', 0.5);
        ////console.log(map.controls)
        var labelDateLeft = labels[0]                 //por default compara la primera y ultima fecha
        var labelDateRight = labels[labels.length - 1]


        //pone las fechas por default en los selectores de fechas
        //document.getElementById("selectLeftCompare").value = labelDateLeft;
        //document.getElementById("selectRightCompare").value = labelDateRight;

        //agrega las capas izquierda/derecha
        selectLeft(labelDateLeft);
        selectRight(labelDateRight);

        //muestra los selectores de fechas y oculta la linea de tiempo
        document.getElementById("selectLeft").style.display = "block";
        document.getElementById("selectRight").style.display = "block";
        document.getElementById("timeLine").style.display = "none";

    }
}

//sale del modo comparacion
function exitCompare() {
    map.getControls().forEach(function (control) {  //checa si ya se añadio el control de comparacion (swipe)
        if (control instanceof ol.control.Swipe) { //ya tiene el control agregado lo quita
            map.removeControl(control);
        }
    });

    //quita las capas de comparacion izquierda/derecha
    var imageLayers = map.getLayers().getArray().slice();
    imageLayers.forEach(layer => {
        if (layer.get('name') === 'left' || layer.get('name') === 'right') {
            map.removeLayer(layer)
        }
    })

    //oculta los selectores de fechas y muestra la linea de tiempo
    document.getElementById("selectLeft").style.display = "none";
    document.getElementById("selectRight").style.display = "none";
    document.getElementById("timeLine").style.display = "block";

    //activa el boton de la grafica
    $('#showGraph').attr('disabled', false);
    $('#chartType').attr('disabled', false);
}


//selecciona el lado izquierdo para comparar
function selectLeft(date) {
    var labelDateLeft = date;
    var layer_type = 'TCI';
    const left = new ol.layer.Tile({
        name: 'left',
        opacity: 1,
        source: new ol.source.XYZ({
            projection: 'EPSG:3857',
            url: '/reports/tiles/' + purchase_id + '/' + zone + '/' + layer_type + '/' + labelDateLeft + '/{z}/{x}/{-y}.png',
            tilePixelRatio: 2,
            minZoom: 10,
            maxZoom: 15
        })
    });
    map.addLayer(left);
    ctrl.addLayer(left);
}

//selecciona el lado derecho para comparar
function selectRight(date) {
    //console.log("---------");
    //console.log(date)
    //console.log("---------");
    var labelDateRight = date;
    var layer_type = 'TCI';
    const right = new ol.layer.Tile({
        name: 'right',
        opacity: 1,
        source: new ol.source.XYZ({
            projection: 'EPSG:3857',
            url: '/reports/tiles/' + purchase_id + '/' + zone + '/' + layer_type + '/' + labelDateRight + '/{z}/{x}/{-y}.png',
            tilePixelRatio: 2,
            minZoom: 10,
            maxZoom: 15
        })
    });
    map.addLayer(right);
    ctrl.addLayer(right, true);
}

//selecciona comparacion vertical u horizontal
function setCompareDirection(direction) {
    ctrl.set('orientation', direction)

    //mueve los selectores de fechas
    if (direction === 'horizontal') {
        document.getElementById("selectLeft").style.bottom = "70%";
        document.getElementById("selectLeft").style.left = "15%";
        document.getElementById("selectRight").style.left = "15%";
    }
    else {
        document.getElementById("selectLeft").style.bottom = "50px";
        document.getElementById("selectLeft").style.left = "30%";
        document.getElementById("selectRight").style.left = "75%";
    }
}

//  Fin funciones de comparacion
//******************************

/************************* Functions de layer Sources ******************************/


function changeMap(selectedMap) {

    sessionStorage.selectedMap = selectedMap;
    map.getLayers().getArray().forEach(function (layer) {
        if (layer.values_.name == "mapbox") {
            map.removeLayer(layer)
        }
    })
    $('#' + selectedMap).prop('checked', true);
    switch (selectedMap) {
        case 'light':
            map.getLayers().insertAt(0, light);
            currentMap = light;
            break;
        case 'streets':
            map.getLayers().insertAt(0, streets);
            currentMap = streets;
            break;
        case 'night':
            map.getLayers().insertAt(0, night);
            currentMap = night;
            break;
        case 'cycle':
            map.getLayers().insertAt(0, cycle);
            currentMap = cycle;
            break;
    }
}

function showControlMap() {

    if ($('#showControlMap').is(':visible')) {
        $("#showControlMap").hide();
        $("#hideControlMap").show();
        // $(".mapControl").show();
    } else {
        $("#showControlMap").show();
        $("#hideControlMap").hide();
        $(".mapControl").hide();
    }
}
